#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.resources module

This module defines a custom adapter which is used to get access to custom CSS and Javascript
resources which can be defined in PyAMS ZMI configuration.

This modules also defines the MyAMS bundle which is used for management interface.
"""

from zope.dublincore.interfaces import IZopeDublinCore
from zope.interface import Interface

from pyams_layer.interfaces import IResources
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.fanstatic import ExternalResource
from pyams_utils.url import absolute_url
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.configuration import IZMIConfiguration, MYAMS_BUNDLES


__docformat__ = 'restructuredtext'


@adapter_config(name='zmi',
                required=(Interface, IAdminLayer, Interface),
                provides=IResources)
class ZMIResourcesAdapter(ContextRequestViewAdapter):
    """ZMI resources adapter"""

    weight = 10

    @property
    def resources(self):
        """Resources getter"""
        request = self.request
        configuration = IZMIConfiguration(request.root, None)
        if configuration is not None:
            # yield main bundle
            bundle, _label = MYAMS_BUNDLES.get(configuration.myams_bundle)
            yield bundle
            # yield stylesheet
            stylesheet = configuration.custom_stylesheet
            if stylesheet:
                modified = IZopeDublinCore(stylesheet).modified
                stylesheet_url = absolute_url(stylesheet, request,
                                              query={'_': modified.timestamp()})  # pylint: disable=no-member
                resource = bundle.library.known_resources.get(stylesheet_url)
                if resource is None:
                    resource = ExternalResource(bundle.library, stylesheet_url,
                                                resource_type='css',
                                                depends=(bundle,))
                yield resource
            # yield script
            script = configuration.custom_script
            if script:
                modified = IZopeDublinCore(script).modified
                script_url = absolute_url(script, request,
                                          query={'_': modified.timestamp()})  # pylint: disable=no-member
                resource = bundle.library.known_resources.get(script_url)
                if resource is None:
                    resource = ExternalResource(bundle.library, script_url,
                                                resource_type='js',
                                                depends=(bundle,))
                yield resource



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.interfaces.table module

This modules defines public interfaces of table views.
"""

__docformat__ = 'restructuredtext'

from zope.contentprovider.interfaces import IContentProvider
from zope.interface import Attribute, Interface
from zope.schema import Bool, TextLine

from pyams_template.template import template_config
from pyams_zmi.interfaces import IInnerAdminView


@template_config(template='templates/table.pt')
@template_config(template='templates/table-empty.pt', name='empty')
class ITableAdminView(IInnerAdminView):
    """Admin table view interface"""

    table_class = Attribute("Inner table class")

    table_label = TextLine(title="Inner table label")


@template_config(template='templates/inner-table.pt')
@template_config(template='templates/inner-table-empty.pt', name='empty')
class IInnerTable(IContentProvider):
    """Inner admin table view interface"""

    table_class = Attribute("Inner table class")

    table_label = TextLine(title="Inner table label")


class ITableElementName(Interface):
    """Table element name interface"""


class ITableWithActions(Interface):
    """Marker interface for table with inner actions menu"""


class ITableElementEditor(Interface):
    """Table row element editor marker interface"""

    view_name = TextLine(title="Editor view name",
                         default='properties.html')

    href = TextLine(title="Editor URL")

    modal_target = Bool(title="Modal target?",
                        required=True,
                        default=True)



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.view module

This module provides generic management views features.
"""

from pyramid.events import subscriber
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty

from pyams_form.interfaces.form import IFormCreatedEvent
from pyams_layer.skin import apply_skin
from pyams_pagelet.interfaces import IPageletCreatedEvent
from pyams_zmi.interfaces import IAdminView, IInnerAdminView, PYAMS_ADMIN_SKIN_NAME


__docformat__ = 'restructuredtext'


@implementer(IAdminView)
class AdminView:
    """Base admin view"""


@implementer(IInnerAdminView)
class InnerAdminView(AdminView):
    """Inner admin view"""

    title = FieldProperty(IInnerAdminView['title'])


@subscriber(IPageletCreatedEvent, context_selector=IAdminView)
def handle_admin_view(event):
    """Automatically apply admin skin to admin views"""
    apply_skin(event.request, PYAMS_ADMIN_SKIN_NAME)


@subscriber(IFormCreatedEvent, context_selector=IAdminView)
def handle_admin_form(event):
    """Automatically apply admin skin to admin forms"""
    apply_skin(event.object.request, PYAMS_ADMIN_SKIN_NAME)



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.interfaces.configuration module

This module defines interfaces of ZMI configuration.
"""

from collections import OrderedDict

from zope.interface import Attribute, Interface
from zope.schema import Bool, Choice, TextLine
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from myams_js import myams_core_svg_bundle, myams_full_bundle, myams_mini_svg_bundle
from pyams_file.schema import FileField, ImageField
from pyams_i18n.schema import I18nTextLineField


__docformat__ = 'restructuredtext'

from pyams_zmi import _


MYAMS_BUNDLES = OrderedDict((
    ('full', (myams_full_bundle, _("MyAMS full bundle"))),
    ('mini+svg', (myams_mini_svg_bundle, _("MyAMS mini bundle (with SVG icons)"))),
    ('core+svg', (myams_core_svg_bundle, _("MyAMS core bundle (with SVG icons)")))
))

MYAMS_BUNDLES_VOCABULARY = SimpleVocabulary(
    [SimpleTerm(k, title=v[1]) for k, v in MYAMS_BUNDLES.items()])


class IZMIConfiguration(Interface):
    """Static management configuration interface"""

    site_name = TextLine(title=_("Site name"),
                         default='PyAMS website',
                         required=False)

    application_name = TextLine(title=_("Application name"),
                                default='PyAMS',
                                required=False)

    application_package = TextLine(title=_("Application package"),
                                   description=_("Name of the main package used to get "
                                                 "application version"),
                                   default='pyams_zmi',
                                   required=False)

    inner_package_name = TextLine(title=_("Secondary name"),
                                  required=False)

    inner_package = TextLine(title=_("Secondary package"),
                             description=_("Name of the secondary package used to complete "
                                           "application version"),
                             required=False)

    version = Attribute("Application version")

    myams_bundle = Choice(title=_("MyAMS bundle"),
                          description=_("MyAMS bundle used by the application"),
                          vocabulary=MYAMS_BUNDLES_VOCABULARY,
                          default='full')

    favicon = ImageField(title=_("Icon"),
                         description=_("Favorites icon"),
                         required=False)

    include_header = Bool(title=_("Include header"),
                          required=False,
                          default=True)

    fixed_header = Bool(title=_("Fixed header"),
                        description=_("If selected, the header will not scroll but will stay "
                                      "fixed at the top of the screen"),
                        required=False,
                        default=True)

    logo = ImageField(title=_("Logo image"),
                      description=_("SVG or bitmap image used as logo"),
                      required=False)

    include_site_search = Bool(title=_("Include site search"),
                               description=_("Include a global site search access link "
                                             "in page header"),
                               required=False,
                               default=False)

    site_search_placeholder = I18nTextLineField(title=_("Site search placeholder"),
                                                required=False)

    site_search_handler = TextLine(title=_("Site search handler"),
                                   required=False,
                                   default='#search.html')

    include_menus = Bool(title=_("Include navigation menus"),
                         required=False,
                         default=True)

    include_minify_button = Bool(title=_("Include minify buttons"),
                                 description=_("If selected, this will provide features to "
                                               "reduce or hide navigation menus"),
                                 required=True,
                                 default=True)

    fixed_navigation = Bool(title=_("Fixed menus"),
                            required=True,
                            default=False)

    accordion_menus = Bool(title=_("Accordion menus"),
                           description=_("If selected, only one navigation menu can be opened "
                                         "at a given time"),
                           required=True,
                           default=True)

    include_ribbon = Bool(title=_("Include ribbon"),
                          description=_("Display breadcrumbs ribbon?"),
                          required=False,
                          default=True)

    fixed_ribbon = Bool(title=_("Fixed ribon"),
                        description=_("If selected, the ribbon will not scroll but will stay "
                                      "fixed at the top of the page"),
                        required=True,
                        default=True)

    base_body_css_class = TextLine(title=_("Base body CSS class"),
                                   required=False)

    body_css_class = Attribute("HTML body CSS class")

    custom_stylesheet = FileField(title=_("Custom stylesheet"),
                                  description=_("Custom stylesheet used to override or extend "
                                                "default MyAMS management skin"),
                                  required=False)

    custom_script = FileField(title=_("Custom script"),
                              description=_("Custom javascript used to override or extend "
                                            "default MyAMS modules"),
                              required=False)



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.form module

This module provides all base form-related classes to be used into PyAMS management
interface.
"""

from pyramid.decorator import reify
from zope.interface import Interface, implementer
from zope.schema.fieldproperty import FieldProperty

from pyams_form.browser.checkbox import SingleCheckBoxFieldWidget
from pyams_form.button import Buttons, handler
from pyams_form.form import AddForm, DisplayForm, EditForm
from pyams_form.group import Group, GroupForm, GroupManager
from pyams_form.interfaces import DISPLAY_MODE
from pyams_form.subform import InnerAddForm, InnerDisplayForm, InnerEditForm
from pyams_i18n.schema import II18nField
from pyams_skin.interfaces.view import IInnerPage, IModalPage
from pyams_utils.data import ObjectDataManagerMixin
from pyams_zmi.interfaces.form import IAddFormButtons, IDisplayFormButtons, IEditFormButtons, \
    IFormGroupChecker, IFormGroupSwitcher, IModalAddFormButtons, IModalDisplayFormButtons, \
    IModalEditFormButtons
from pyams_zmi.view import AdminView


__docformat__ = 'restructuredtext'


#
# Base add forms
#

# pylint: disable=abstract-method
@implementer(IInnerPage)
class AdminAddForm(ObjectDataManagerMixin, GroupForm, AddForm, AdminView):
    """Management add form"""

    @property
    def buttons(self):
        """Default add form buttons getter"""
        if self.mode == DISPLAY_MODE:
            return Buttons(Interface)
        return Buttons(IAddFormButtons)

    @handler(IAddFormButtons['add'])
    def handle_add(self, action):
        """Default add form button handler"""
        super(AdminAddForm, self).handle_add(self, action)  # pylint: disable=too-many-function-args


@implementer(IModalPage)
class AdminModalAddForm(AdminAddForm):
    """Modal management add form"""

    @property
    def buttons(self):
        if self.mode == DISPLAY_MODE:
            return Buttons(IModalDisplayFormButtons)
        return Buttons(IModalAddFormButtons)

    modal_class = FieldProperty(IModalPage['modal_class'])
    ajax_form_target = None

    @handler(IModalAddFormButtons['add'])
    def handle_add(self, action):
        super(AdminModalAddForm, self).handle_add(self, action)  # pylint: disable=too-many-function-args


@implementer(IInnerPage)
class AdminInnerAddForm(ObjectDataManagerMixin, GroupForm, InnerAddForm, AdminView):
    """Inner management add form"""

    buttons = Buttons(Interface)


#
# Base edit forms
#

@implementer(IInnerPage)
class AdminEditForm(ObjectDataManagerMixin, GroupForm, EditForm, AdminView):
    """Management edit form"""

    @property
    def buttons(self):
        """Default inner page buttons getter"""
        if self.mode == DISPLAY_MODE:
            return Buttons(Interface)
        return Buttons(IEditFormButtons)

    @handler(IEditFormButtons['apply'])
    def handle_apply(self, action):
        super(AdminEditForm, self).handle_apply(self, action)  # pylint: disable=too-many-function-args


@implementer(IModalPage)
class AdminModalEditForm(AdminEditForm):
    """Modal management edit form"""

    @property
    def buttons(self):
        if self.mode == DISPLAY_MODE:
            return Buttons(IModalDisplayFormButtons)
        return Buttons(IModalEditFormButtons)

    modal_class = FieldProperty(IModalPage['modal_class'])
    ajax_form_target = None

    @handler(IModalEditFormButtons['apply'])
    def handle_apply(self, action):
        super(AdminModalEditForm, self).handle_apply(self, action)  # pylint: disable=too-many-function-args


@implementer(IInnerPage)
class AdminInnerEditForm(ObjectDataManagerMixin, GroupForm, InnerEditForm, AdminView):
    """Inner management edit form"""

    buttons = Buttons(Interface)


#
# Base display forms
#

@implementer(IInnerPage)
class AdminDisplayForm(ObjectDataManagerMixin, GroupManager, DisplayForm, AdminView):
    """Management display form"""

    buttons = Buttons(IDisplayFormButtons)


@implementer(IModalPage)
class AdminModalDisplayForm(AdminDisplayForm):
    """Modal management display form"""

    buttons = Buttons(IModalDisplayFormButtons)

    modal_class = FieldProperty(IModalPage['modal_class'])


@implementer(IInnerPage)
class AdminInnerDisplayForm(ObjectDataManagerMixin, GroupForm, InnerDisplayForm, AdminView):
    """Inner management display form"""

    buttons = Buttons(Interface)


#
# Switcher group
#

@implementer(IFormGroupSwitcher)
class FormGroupSwitcher(ObjectDataManagerMixin, Group):
    """Form group switcher

    A "group switcher" is based on a "switcher" component provided by MyAMS, which allows to
    switch a whole fieldset.
    """

    minus_class = FieldProperty(IFormGroupSwitcher['minus_class'])
    plus_class = FieldProperty(IFormGroupSwitcher['plus_class'])
    switcher_mode = FieldProperty(IFormGroupSwitcher['switcher_mode'])

    @property
    def state(self):
        """Current state getter"""
        if self.switcher_mode == 'always':
            return 'open'
        if self.switcher_mode == 'never':
            return 'closed'
        # else: automatic mode
        for widget in self.widgets.values():
            if widget.ignore_context:
                continue
            field = widget.field
            if self.ignore_context:
                value = field.default
            else:
                context = widget.context
                name = field.getName()
                value = getattr(field.interface(context), name, None)
            if value and (value != field.default):
                if II18nField.providedBy(field):
                    for i18n_value in value.values():
                        if i18n_value:
                            return 'open'
                return 'open'
        return 'closed'


#
# Checker group
#

@implementer(IFormGroupChecker)
class FormGroupChecker(ObjectDataManagerMixin, Group):
    """Form group checker

    A "group checker" is based on a "checker" component provided by MyAMS, which allows to
    hide or disable a whole fieldset with a checkbox matching a form's field value.
    """

    checker_fieldname = FieldProperty(IFormGroupChecker['checker_fieldname'])
    checker_mode = FieldProperty(IFormGroupChecker['checker_mode'])

    def __init__(self, context, request, parent_form):
        super(FormGroupChecker, self).__init__(context, request, parent_form)
        name, field = next(iter(self.fields.items()))
        self.checker_fieldname = name
        self.legend = field.field.title
        self.fields[self.checker_fieldname].widget_factory = SingleCheckBoxFieldWidget

    @reify
    def checker_widget(self):
        """Checker widget getter"""
        return self.widgets[self.checker_fieldname]

    @property
    def checker_state(self):
        """Checker state getter"""
        return 'on' if 'selected' in self.checker_widget.value else 'off'



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.zmi.viewlet.ribbon module

This modules defines "pyams.page_ribbon", which is used to build PyAMS breadcrumbs ribbon of
the management interface.
"""

from pyams_template.template import template_config
from pyams_viewlet.viewlet import ViewContentProvider, contentprovider_config
from pyams_zmi.interfaces import IAdminLayer


__docformat__ = 'restructuredtext'


@contentprovider_config(name='pyams.page_ribbon', layer=IAdminLayer)
@template_config(template='templates/page-ribbon.pt')
class RibbonContentProvider(ViewContentProvider):
    """Ribbon content provider"""



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.zmi.viewlet.toplinks module

This module provides "pyams.toplinks" viewlet manager and a small set of components which are
used to display tabs and links in PyAMS management interface.
"""

from pyams_template.template import template_config
from pyams_utils.data import ObjectDataManagerMixin
from pyams_viewlet.manager import TemplateBasedViewletManager, WeightOrderedViewletManager,\
    viewletmanager_config
from pyams_viewlet.viewlet import Viewlet
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IPageHeaderViewletManager, ITopLinksViewletManager


__docformat__ = 'restructuredtext'


@viewletmanager_config(name='pyams.toplinks', layer=IAdminLayer,
                       manager=IPageHeaderViewletManager, weight=100,
                       provides=ITopLinksViewletManager)
@template_config(template='templates/top-links.pt')
class TopLinksViewletManager(TemplateBasedViewletManager, WeightOrderedViewletManager):
    """Top links viewlet manager"""


@template_config(template='templates/top-menu.pt')
class TopMenuViewletManager(TemplateBasedViewletManager, WeightOrderedViewletManager):
    """Top menu viewlet manager"""

    min_display_size = ''
    """Minimum media display size; can be 'sm', 'md'..."""

    label = 'Menu label'
    """Menu label"""

    selection = '(selected value)'
    """Displayed menu selection"""


@template_config(template='templates/top-menu-item.pt')
class TopMenuItem(ObjectDataManagerMixin, Viewlet):
    """Top menu item"""

    label = 'Menu item label'
    """Menu item label"""

    href = '/'
    """Menu URL target"""

    css_class = 'dropdown-item'
    """CSS class"""


@template_config(template='templates/top-tabs.pt')
class TopTabsViewletManager(TemplateBasedViewletManager, WeightOrderedViewletManager):
    """Top tabs viewlet manager"""

    min_display_size = ''
    """Minimum media display size; can be 'sm', 'md'..."""

    label = 'Tabs label'
    """Tabs group label"""


@template_config(template='templates/top-tab.pt')
class TopTabItem(ObjectDataManagerMixin, Viewlet):
    """Top tab item base class"""

    label = 'Tab item label'
    """Tab item label"""

    href = '#content'
    """Tab content target"""

    url = '/admin'
    """URL pointed by the tab"""



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.search module

This module provides a little set of base classes which can be used in search forms.
"""

from pyramid.interfaces import IView
from zope.interface import Interface, implementer

from pyams_form.button import Buttons
from pyams_form.field import Fields
from pyams_pagelet.pagelet import Pagelet
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import get_object_factory, is_interface
from pyams_zmi.form import AdminAddForm
from pyams_zmi.interfaces import IAdminLayer, IPageTitle
from pyams_zmi.interfaces.form import ISearchButtons, ISearchInfo, ISearchView
from pyams_zmi.table import TableAdminView
from pyams_zmi.view import InnerAdminView


__docformat__ = 'restructuredtext'

from pyams_zmi import _  # pylint: disable=ungrouped-imports


# pylint: disable=abstract-method
class SearchForm(AdminAddForm):
    """Base search form"""

    title = _("Search form")
    legend = _("Search criteria")

    fields = Fields(ISearchInfo)
    buttons = Buttons(ISearchButtons)

    ajax_form_handler = 'search-results.html'
    ajax_form_target = '#search-results'


@implementer(ISearchView)
class SearchView(InnerAdminView):
    """Base search view"""

    search_form = SearchForm

    def __init__(self, context, request):
        super(SearchView, self).__init__(context, request)
        factory = get_object_factory(self.search_form) if is_interface(self.search_form) \
            else self.search_form
        self.search_form = factory(context, request)

    def update(self):
        """Search view update"""
        super(SearchView, self).update()  # pylint: disable=no-member
        self.search_form.update()


@implementer(IView)
class SearchResultsView(TableAdminView, Pagelet):
    """Search results view"""

    title = None


@adapter_config(required=(Interface, IAdminLayer, SearchResultsView),
                provides=IPageTitle)
def search_results_title(context, request, view):  # pylint: disable=unused-argument
    """Search results view title adapter"""
    return ' '



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.interfaces.form module

This module provides public interfaces for management forms.
"""

from zope.interface import Attribute, Interface
from zope.schema import Choice, TextLine

from pyams_form.interfaces.form import IDisplayForm, IForm, IGroup, IInnerSubForm, IInnerTabForm
from pyams_skin.schema.button import CloseButton, ResetButton, SubmitButton
from pyams_template.template import override_template, template_config
from pyams_zmi.interfaces import IAdminLayer


__docformat__ = 'restructuredtext'

from pyams_zmi import _


override_template(IForm, template='templates/form.pt', layer=IAdminLayer)
override_template(IDisplayForm, template='templates/form-display.pt', layer=IAdminLayer)

override_template(IGroup, template='templates/form-group.pt', layer=IAdminLayer)

override_template(IInnerSubForm, template='templates/form-group.pt', layer=IAdminLayer)

override_template(IInnerTabForm, template='templates/form-tabform.pt', layer=IAdminLayer)


class IAddFormButtons(Interface):
    """Add forms buttons interface"""

    add = SubmitButton(name='add',
                       title=_("Add"))

    cancel = ResetButton(name='reset',
                         title=_("Reset"))


class IModalAddFormButtons(Interface):
    """Modal add forms buttons interface"""

    add = SubmitButton(name='add',
                       title=_("Add"))

    close = CloseButton(name='close',
                        title=_("Cancel"))


class IEditFormButtons(Interface):
    """Edit forms buttons interface"""

    apply = SubmitButton(name='apply',
                         title=_("Apply"))

    cancel = ResetButton(name='reset',
                         title=_("Reset"))


class IModalEditFormButtons(Interface):
    """Modal edit forms buttons interface"""

    apply = SubmitButton(name='apply',
                         title=_("Apply"))

    close = CloseButton(name='close',
                        title=_("Cancel"))


class IDisplayFormButtons(Interface):
    """Display forms buttons interface"""


class IModalDisplayFormButtons(Interface):
    """Modal display forms buttons interface"""

    close = CloseButton(name='close',
                        title=_("Cancel"))


@template_config(template='templates/form-switcher.pt', layer=IAdminLayer)
class IFormGroupSwitcher(IGroup):
    """Form group switcher interface"""

    minus_class = TextLine(title="Expanded switcher FontAwesome CSS class (without prefix)",
                           default='minus')

    plus_class = TextLine(title="Reduced switcher FontAwesome CSS class (without prefix)",
                          default='plus')

    switcher_mode = Choice(title="Switcher display mode",
                           values=('always', 'never', 'auto'),
                           default='auto')

    state = Attribute("Initial switcher state")


@template_config(template='templates/form-checker.pt', layer=IAdminLayer)
class IFormGroupChecker(IGroup):
    """Form group checker interface"""

    checker_fieldname = TextLine(title="Checker checkbox field name")

    checker_widget = Attribute("Checker checkbox field")

    checker_mode = Choice(title="Checker display mode",
                          values=('hide', 'disable'),
                          default='hide')

    checker_state = Attribute("Initial switcher state")


@template_config(template='templates/search-view.pt', layer=IAdminLayer)
class ISearchView(Interface):
    """Base search view interface"""

    search_factory = Attribute("Search form interface")

    search_form = Attribute("Search form view")


class ISearchInfo(Interface):
    """Base search interface"""

    query = TextLine(title=_("Search for"))


class ISearchButtons(Interface):
    """Search form buttons interface"""

    search = SubmitButton(name='search', title=_("Search"))

    cancel = ResetButton(name='reset', title=_("Reset"))



#
# Copyright (c) 2015-2020 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_zmi.zmi.viewlet.logo module

"pyams.logo" is a viewlet which is used to display a logo in PyAMS top panel.
This logo is extracted from ZMI configuration.
"""

__docformat__ = 'restructuredtext'

from pyams_template.template import template_config
from pyams_viewlet.viewlet import Viewlet, viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.configuration import IZMIConfiguration
from pyams_zmi.interfaces.viewlet import IPageHeaderViewletManager


@viewlet_config(name='pyams.logo', layer=IAdminLayer,
                manager=IPageHeaderViewletManager, weight=0)
@template_config(template='templates/logo.pt')
class LogoViewlet(Viewlet):
    """Logo viewlet"""

    href = '/admin'

    @property
    def logo(self):
        """Logo getter"""
        configuration = IZMIConfiguration(self.request.root, None)
        if not configuration:
            return None
        return configuration.logo
